@@ -50,7 +50,7 @@ from calculate.lib.utils.portage import (Layman, EmergeLog,
                                          get_packages_files_directory,
                                          get_manifest_files_directory,
                                          get_remove_list)
-from calculate.lib.utils.text import _u8
+from calculate.lib.utils.text import _u8, _u
 
 Colors = TextState.Colors
 from calculate.lib.utils.files import (getProgPath, STDOUT, removeDir,
@@ -629,10 +629,14 @@ class Update(MethodsInterface):
         if not name.strip():
             name = str(pkg)
 
+        
+        name = str(pkg)
+        name = _u(name)
+
         self.printSUCCESS(
-            _("{part} {package}").format(part=part, package=name))
+            _u(_("{part} {package}")).format(part=_u(part), package=name))
         self.startTask(
-            _("Emerging {package}").format(package=_colorprint(str(pkg))))
+            _u(_("Emerging {package}")).format(package=_u(_colorprint(str(pkg)))))
 
     def _printInstallPackage(self, pkg, binary=False):
         """
